/* -*- Javascript -*- */

/* global variables */

var config = null;
var file = null;
var export_filename = 'export_config.xml';

/* add css rule */

var acr_sheet = (function() {
  try {
    var style = document.createElement('style');
    style.appendChild(document.createTextNode(''));
    document.head.appendChild(style);
    return style.sheet;
  }
  catch (err) {
    return null;
  }
})();
var acr_index = 0;
function add_css_rule(selector, rules) {
  if (acr_sheet == null)
    return;
  else if("insertRule" in acr_sheet)
    acr_sheet.insertRule(selector + "{" + rules + "}", acr_index++);
  else if("addRule" in acr_sheet)
    acr_sheet.addRule(selector, rules, acr_index++);
}

/* load function */

function load_config () {

  /* update status */
  update_status (get_message ('status_load_config'), 0);

  /* create request message */
  var params = 'action=load';
  var rxfunc = function (message) {
    var stat = message.getElementsByTagName ('stat')[0].firstChild.nodeValue;
    if (stat == 0) {
      var data = message.getElementsByTagName ('data')[0].firstChild.nodeValue;
      config = base64_decode (data);
      try { config = decodeURIComponent (escape (config)); } catch (err) { }
      /* special case to  customize elements in page */
      var tab = config.replace (/\\\n/g, '').split ('\n');
      for (var i in tab) {
        var row = tab[i].replace (/^\s*#.*/, '').split (';');
        if (row.length < 3) continue;
        var element_list = row[0].split (',');
        var description = row[2];
        if (element_list.length == 1) {
          if (element_list[0] == ':title:') {
            document.title = description;
            var h1 = document.getElementsByTagName ('h1')[0];
           if (h1) h1.innerHTML = description;
          } else {
            var ref = element_list[0].split (':');
            if ((ref.length == 3) && (ref[0] == '') && (ref[2] == '')) {
	      description = row.slice (2).join (';').replace (/;;$/, '');
              if (ref[1] == '') {
                var rule = description.replace (/}/, '').split ('{');
                add_css_rule ((rule.length > 1) ? rule[0] : '',
                            (rule.length > 1) ? rule[1] : rule[0]);
              } else {
                var elem = document.getElementById (ref[1]);
                if (elem) elem.innerHTML = description;
              }
            }
          }
        }
      }
    } else {

      /* update status on error */
      update_status (get_message ('status_load_error', stat), 2);
    }
  };

  ajax_post (params, rxfunc, 'cfg/func.php');

  return '';
}

/* create row in table */

function create_row (table, label, variable, value, type, default_value) {
  if (typeof type === 'undefined') type = 'string';
  if (typeof default_value === 'undefined') default_value = '';

  /* create row */
  var row_config = table.insertRow (table.rows.length);

  if (variable == '') {

    /* title */
    var cell_title_config = row_config.insertCell (0);
    cell_title_config.colSpan = 2;
    var title_config = document.createElement ('h3');
    title_config.innerHTML = label;
    cell_title_config.appendChild (title_config);

  } else {

    /* create label column */
    var cell_label_config = row_config.insertCell (0);
    cell_label_config.className = 'fit';
    var label_config = document.createElement ('label');
    label_config.innerHTML = label + ': ';
    label_config.forName = variable;
    cell_label_config.appendChild (label_config);

    /* create input column */
    var cell_input_config = row_config.insertCell (1);
    if (type == 'string')
      cell_input_config.className = 'expand';
    var input_config = create_input (variable, value, type, default_value);
    cell_input_config.appendChild (input_config);

  }

  return table;
}

/* create full table */

function create_table (xmltree, config) {

  var table = document.createElement ('table');
  table.className = 'table';
  table.id = 'config';

  var tab = config.split ('\n');
  for (var i in tab) {

    var row = tab[i].replace (/#.*/, '').split (';');
    if (row.length <= 1)
      continue;
    var element_list = row[0].split (',');
    var param = row[1];
    var description = row[2];
    var type = row[3];
    var default_value = row[4];

    if ((type == '') || (row[0] == '::')) {

      if (!element_list[0].match (/^:.*:$/))
        create_row (table, description, '', '', '', '');

      continue;
    }

    var name = get_tree_name (xmltree, element_list);
    if (!name)
      continue;
    name += param;

    var elements = find_element_from_xmltree (xmltree, element_list.slice (0));
    if (!elements)
      continue;

    for (var j in elements) {
      var element = elements[j];

      var value = (param.length) ? element.getAttribute (param) : element.textContent;
      if (typeof value === 'undefined')
        continue;

      create_row (table, description, name + '_' + j, value, type, default_value);
    }
  }

  return table;
}

/* update xmltree */

function update_xmltree (xmltree, config) {

  if (!config)
    return;

  var tab = config.split ('\n');
  for (var i in tab) {
 
    var row = tab[i].replace (/^\s*#.*/, '').split (';');
    if (row.length <= 1)
      continue;
    var element_list = row[0].split (',');
    var param = row[1];
    var type = row[3];

    if ((element_list.length == 1) && ((element_list[0].match (/^:.*:$/)) || (element_list[0] == '')))
      continue;

    var name = get_tree_name (xmltree, element_list);
    if (!name)
      continue;
    name += param;

    var elements = find_element_from_xmltree (xmltree, element_list.slice (0));
    if (!elements)
      continue;
 
    for (var j in elements) {
      var element = elements[j];

      var ref = document.getElementById (name + '_' + j);
      if (ref) {
        var value = get_input_value (ref);
        if (param.length)
          element.setAttribute (param, value);
        else
          element.textContent = value;
      }
    }
  }
}

/* import procedure */

function import_proc (data) {

  /* remove previous table */
  var table = document.getElementById ('config');
  if (!table) {
    update_status (get_message ('status_table_error'));
    return;
  }
  var father = table.parentNode;
  father.removeChild (table);

  /* save data file*/
  data = data.replace (/\r/g, '\n').replace (/\n+/g, '\n');
  file = data;

  /* process xml data */
  var xmldata;
  if (window.DOMParser) {
    xmldata = (new DOMParser ()).parseFromString (file, 'text/xml');
  } else { // code for IE
    xmldata = new ActiveXObject ('Microsoft.XMLDOM');
    xmldata.async = false;
    xmldata.loadXML (file);
  }

  /* create new table */
  table = create_table (xmldata, config);

  /* add new table into html page */
  father.appendChild (table);

  /* table is updated */
  update_status (get_message ('status_import_ok'), 0);
}

/* export raw database */

function export_proc () {

  /* look for table */
  var table = document.getElementById ('config');
  if (!table) {
    update_status (get_message ('status_table_error'));
    return;
  }

  /* process xml data */
  var xmldata;
  if (window.DOMParser) {
    xmldata = (new DOMParser ()).parseFromString (file, 'text/xml');
  } else {
    xmldata = new ActiveXObject ('Microsoft.XMLDOM');
    xmldata.async = false;
    if (file) xmldata.loadXML (file);
  }

  /*  process element/argument tuplet */
  update_xmltree (xmldata, config);

  /* serialize data */
  var data = '';
  if (window.XMLSerializer) {
    data = (new XMLSerializer ()).serializeToString (xmldata.documentElement);
  } else if (typeof xmldata.xml !== 'undefined') { // code for IE
    data = xmldata.xml;
  }
  data = data.replace (/\r/g, '\n').replace (/\n+/g, '\n');

  /* add static header */
  var header = '<?xml version="1.0" encoding="ISO-8859-1"?>\n';

  return (data.indexOf ('<?xml') !== 0) ? header + data : data;
}
